@@ -20,18 +20,6 @@
 
 static ACQUISITIONCAPI AcquisitionCAPI;
 
-// Py_XSETREF is undefined in Python 3.4 only, it's present in 2.7 and 3.5
-#ifndef Py_XSETREF
-
-#define Py_XSETREF(op, op2)                     \
-    do {                                        \
-        PyObject *_py_tmp = (PyObject *)(op);   \
-        (op) = (op2);                           \
-        Py_XDECREF(_py_tmp);                    \
-    } while (0)
-
-#endif
-
 #define ASSIGN(dst, src) Py_XSETREF(dst, src)
 #define OBJECT(O) ((PyObject*)(O))
 
@@ -46,7 +34,8 @@ static PyObject *py__add__, *py__sub__, *py__mul__, *py__div__,
   *py__long__, *py__float__, *py__oct__, *py__hex__,
   *py__getitem__, *py__setitem__, *py__delitem__,
   *py__getslice__, *py__setslice__, *py__delslice__,  *py__contains__,
-  *py__len__, *py__of__, *py__call__, *py__repr__, *py__str__, *py__unicode__,
+  *py__len__, *py__of__, *py__call__, *py__repr__,
+  *py__str__, *py__unicode__, *py__bytes__,
   *py__cmp__, *py__parent__, *py__iter__, *py__bool__, *py__index__, *py__iadd__,
   *py__isub__, *py__imul__, *py__imod__, *py__ipow__, *py__ilshift__, *py__irshift__,
   *py__iand__, *py__ixor__, *py__ior__, *py__floordiv__, *py__truediv__,
@@ -95,6 +84,7 @@ init_py_names(void)
   INIT_PY_NAME(__repr__);
   INIT_PY_NAME(__str__);
   INIT_PY_NAME(__unicode__);
+  INIT_PY_NAME(__bytes__);
   INIT_PY_NAME(__cmp__);
   INIT_PY_NAME(__parent__);
   INIT_PY_NAME(__iter__);
@@ -970,6 +960,20 @@ Wrapper_unicode(Wrapper *self)
     }
 }
 
+static PyObject *
+Wrapper_bytes(Wrapper *self)
+{
+    PyObject *r;
+
+    if ((r = PyObject_GetAttr(OBJECT(self), py__bytes__))) {
+        ASSIGN(r, PyObject_CallFunction(r, NULL, NULL));
+        return r;
+    } else {
+        PyErr_Clear();
+        return PyBytes_FromObject(self->obj);
+    }
+}
+
 static long
 Wrapper_hash(Wrapper *self)
 {
@@ -1197,11 +1201,6 @@ static PyMappingMethods Wrapper_as_mapping = {
 
 WRAP_BINOP(sub);
 WRAP_BINOP(mul);
-
-#ifndef PY3K
-WRAP_BINOP(div);
-#endif
-
 WRAP_BINOP(mod);
 WRAP_BINOP(divmod);
 WRAP_TERNARYOP(pow);
@@ -1216,26 +1215,11 @@ WRAP_BINOP(xor);
 WRAP_BINOP(or);
 
 WRAP_UNARYOP(int);
-
-#ifndef PY3K
-WRAP_UNARYOP(long);
-#endif
-
 WRAP_UNARYOP(float);
 
-#ifndef PY3K
-WRAP_UNARYOP(oct);
-WRAP_UNARYOP(hex);
-#endif
-
 WRAP_BINOP(iadd);
 WRAP_BINOP(isub);
 WRAP_BINOP(imul);
-
-#ifndef PY3K
-WRAP_BINOP(idiv);
-#endif
-
 WRAP_BINOP(imod);
 WRAP_TERNARYOP(ipow);
 WRAP_BINOP(ilshift);
@@ -1261,11 +1245,7 @@ Wrapper_nonzero(PyObject *self)
     PyObject* result = NULL;
     PyObject* callable = NULL;
 
-#ifdef PY3K
     callable = PyObject_GetAttr(self, py__bool__);
-#else
-    callable = PyObject_GetAttr(self, py__nonzero__);
-#endif
 
     if (callable == NULL) {
         PyErr_Clear();
@@ -1291,43 +1271,11 @@ Wrapper_nonzero(PyObject *self)
 
 }
 
-#ifndef PY3K
-static int
-Wrapper_coerce(PyObject **self, PyObject **o)
-{
-    PyObject *m;
-
-    if ((m=PyObject_GetAttr(*self, py__coerce__)) == NULL) {
-        PyErr_Clear();
-        Py_INCREF(*self);
-        Py_INCREF(*o);
-        return 0;
-    }
-
-    ASSIGN(m, PyObject_CallFunction(m, "O", *o));
-    if (m == NULL) {
-        return -1;
-    }
-
-    if (!PyArg_ParseTuple(m, "OO", self, o)) {
-        Py_DECREF(m);
-        return -1;
-    }
-
-    Py_INCREF(*self);
-    Py_INCREF(*o);
-    Py_DECREF(m);
-    return 0;
-}
-#endif
 
 static PyNumberMethods Wrapper_as_number = {
     Wrapper_add,        /* nb_add */
     Wrapper_sub,        /* nb_subtract */
     Wrapper_mul,        /* nb_multiply */
-#ifndef PY3K
-    Wrapper_div,        /* nb_divide */
-#endif
     Wrapper_mod,        /* nb_remainder */
     Wrapper_divmod,     /* nb_divmod */
     Wrapper_pow,        /* nb_power */
@@ -1341,34 +1289,12 @@ static PyNumberMethods Wrapper_as_number = {
     Wrapper_and,        /* nb_and */
     Wrapper_xor,        /* nb_xor */
     Wrapper_or,         /* nb_or */
-
-#ifndef PY3K
-    Wrapper_coerce,     /* nb_coerce */
-#endif
-
     Wrapper_int,        /* nb_int */
-
-#ifdef PY3K
     NULL,
-#else
-    Wrapper_long,       /* nb_long */
-#endif
-
     Wrapper_float,      /* nb_float */
-
-#ifndef PY3K
-    Wrapper_oct,        /* nb_oct*/
-    Wrapper_hex,        /* nb_hex*/
-#endif
-
     Wrapper_iadd,       /* nb_inplace_add */
     Wrapper_isub,       /* nb_inplace_subtract */
     Wrapper_imul,       /* nb_inplace_multiply */
-
-#ifndef PY3K
-    Wrapper_idiv,       /* nb_inplace_divide */
-#endif
-
     Wrapper_imod,       /* nb_inplace_remainder */
     Wrapper_ipow,       /* nb_inplace_power */
     Wrapper_ilshift,    /* nb_inplace_lshift */
@@ -1487,6 +1413,8 @@ static struct PyMethodDef Wrapper_methods[] = {
    "Wrappers are not picklable"},
   {"__unicode__", (PyCFunction)Wrapper_unicode, METH_NOARGS,
    "Unicode"},
+  {"__bytes__", (PyCFunction)Wrapper_bytes, METH_NOARGS,
+   "Bytes"},
   {NULL,  NULL}
 };
 
@@ -1981,7 +1909,6 @@ static struct PyMethodDef methods[] = {
   {NULL,	NULL}
 };
 
-#ifdef PY3K
 static struct PyModuleDef moduledef =
 {
     PyModuleDef_HEAD_INIT,
@@ -1994,7 +1921,6 @@ static struct PyModuleDef moduledef =
     NULL,                                   /* m_clear */
     NULL,                                   /* m_free */
 };
-#endif
 
 
 static PyObject*
@@ -2022,14 +1948,7 @@ module_init(void)
         return NULL;
     }
 
-#ifdef PY3K
     m = PyModule_Create(&moduledef);
-#else
-    m = Py_InitModule3("_Acquisition",
-                       methods,
-                       "Provide base classes for acquiring objects\n\n");
-#endif
-
     d = PyModule_GetDict(m);
     init_py_names();
     PyExtensionClass_Export(d,"Acquirer", AcquirerType);
@@ -2059,14 +1978,7 @@ module_init(void)
     return m;
 }
 
-#ifdef PY3K
 PyMODINIT_FUNC PyInit__Acquisition(void)
 {
     return module_init();
 }
-#else
-PyMODINIT_FUNC init_Acquisition(void)
-{
-    module_init();
-}
-#endif
