@@ -14,7 +14,6 @@
 """Acquisition test cases (and useful examples)
 """
 
-from __future__ import print_function
 import gc
 import unittest
 import sys
@@ -35,25 +34,24 @@ from Acquisition import (  # NOQA
     aq_self,
     Explicit,
     Implicit,
+    CAPI,
     IS_PYPY,
     IS_PURE,
+    _Wrapper,
 )
 
-if sys.version_info >= (3,):
-    PY3 = True
-    PY2 = False
 
-    def unicode(self):
-        # For test purposes, redirect the unicode
-        # to the type of the object, just like Py2 did
-        try:
-            return type(self).__unicode__(self)
-        except AttributeError as e:
-            return type(self).__str__(self)
-    long = int
-else:
-    PY2 = True
-    PY3 = False
+def unicode(self):
+    # For test purposes, redirect the unicode
+    # to the type of the object, just like Py2 did
+    try:
+        return type(self).__unicode__(self)
+    except AttributeError:
+        return type(self).__str__(self)
+
+def cmp(x, y):
+    return 0 if x == y else -1 if x < y else 1
+
 
 if 'Acquisition._Acquisition' not in sys.modules:
     CAPI = False
@@ -62,17 +60,17 @@ else:
 
 MIXIN_POST_CLASS_DEFINITION = True
 try:
-    class Plain(object):
+    class Plain:
         pass
     Plain.__bases__ = (ExtensionClass.Base, )
 except TypeError:
     # Not supported
     MIXIN_POST_CLASS_DEFINITION = False
 
-AQ_PARENT = unicode('aq_parent')
-UNICODE_WAS_CALLED = unicode('unicode was called')
-STR_WAS_CALLED = unicode('str was called')
-TRUE = unicode('True')
+AQ_PARENT = 'aq_parent'
+UNICODE_WAS_CALLED = 'unicode was called'
+STR_WAS_CALLED = 'str was called'
+TRUE = 'True'
 
 
 class I(Implicit):
@@ -93,7 +91,7 @@ class E(Explicit):
         return self.id
 
 
-class Location(object):
+class Location:
     __parent__ = None
 
 
@@ -292,16 +290,16 @@ class TestStory(unittest.TestCase):
         # - The object found, and
         # - The extra data passed to 'aq_acquire'.
 
-        # If the filter returns a true object that the object found is
+        # If the filter returns a true object then the object found is
         # returned, otherwise, the acquisition search continues.
         # For example, in:
 
-        class HandyForTesting(object):
+        class HandyForTesting:
             def __init__(self, name):
                 self.name = name
 
             def __str__(self):
-                return "%s(%s)" % (self.name, self.__class__.__name__)
+                return f"{self.name}({self.__class__.__name__})"
 
             __repr__ = __str__
 
@@ -355,7 +353,7 @@ class TestStory(unittest.TestCase):
                 self.name = name
 
             def __str__(self):
-                return "%s(%s)" % (self.name, self.__class__.__name__)
+                return f"{self.name}({self.__class__.__name__})"
 
             __repr__ = __str__
 
@@ -1394,7 +1392,7 @@ class TestPickle(unittest.TestCase):
     def test_cant_pickle_acquisition_wrappers_classic(self):
         import pickle
 
-        class X(object):
+        class X:
             def __getstate__(self):
                 return 1
 
@@ -1431,7 +1429,7 @@ class TestPickle(unittest.TestCase):
     def test_cant_pickle_acquisition_wrappers_newstyle(self):
         import pickle
 
-        class X(object):
+        class X:
             def __getstate__(self):
                 return 1
 
@@ -1471,7 +1469,7 @@ class TestPickle(unittest.TestCase):
         except ImportError:
             import pickle as cPickle
 
-        class X(object):
+        class X:
             _p_oid = '1234'
 
             def __getstate__(self):
@@ -1519,7 +1517,7 @@ class TestPickle(unittest.TestCase):
             pickler.inst_persistent_id = persistent_id
         except AttributeError:
             pass
-        pickler.persistent_id = persistent_id  # PyPy and Py3k
+        pickler.persistent_id = persistent_id
         pickler.dump(w)
         state = file.getvalue()
         self.assertTrue(b'1234' in state)
@@ -1531,7 +1529,7 @@ class TestPickle(unittest.TestCase):
         except ImportError:
             import pickle as cPickle
 
-        class X(object):
+        class X:
             _p_oid = '1234'
 
             def __getstate__(self):
@@ -1578,7 +1576,7 @@ class TestPickle(unittest.TestCase):
         except AttributeError:
             pass
 
-        pickler.persistent_id = persistent_id  # PyPy and Py3k
+        pickler.persistent_id = persistent_id
         pickler.dump(w)
         state = file.getvalue()
         self.assertTrue(b'1234' in state)
@@ -1624,7 +1622,7 @@ class TestMixin(unittest.TestCase):
         # but also doesn't result in any wrappers.
         from ExtensionClass import Base
 
-        class Plain(object):
+        class Plain:
             pass
 
         self.assertEqual(Plain.__bases__, (object, ))
@@ -1653,7 +1651,7 @@ class TestMixin(unittest.TestCase):
         # We can mix-in Base as part of multiple inheritance.
         from ExtensionClass import Base
 
-        class MyBase(object):
+        class MyBase:
             pass
 
         class MixedIn(Base, MyBase):
@@ -1727,7 +1725,7 @@ class TestGC(unittest.TestCase):
         for B in I, E:
             counter = [0]
 
-            class C(object):
+            class C:
                 def __del__(self, counter=counter):
                     counter[0] += 1
 
@@ -1754,7 +1752,7 @@ def test_container_proxying():
     ...     def __getitem__(self, key):
     ...         if isinstance(key, slice):
     ...             print('slicing...')
-    ...             return (key.start,key.stop)
+    ...             return (key.start, key.stop, key.step)
     ...         print('getitem', key)
     ...         if key == 4:
     ...             raise IndexError
@@ -1767,7 +1765,7 @@ def test_container_proxying():
     ...         return iter((42,))
     ...     def __getslice__(self, start, end):
     ...         print('slicing...')
-    ...         return (start, end)
+    ...         return (start, end, None)
 
     The naked class behaves like this:
 
@@ -1783,8 +1781,17 @@ def test_container_proxying():
     [42]
     >>> c[5:10]
     slicing...
-    (5, 10)
-    >>> c[5:] == (5, sys.maxsize if PY2 else None)
+    (5, 10, None)
+    >>> c[5:] == (5, None, None)
+    slicing...
+    True
+    >>> c[:10] == (None, 10, None)
+    slicing...
+    True
+    >>> c[5:10:5] == (5, 10, 5)
+    slicing...
+    True
+    >>> c[::] == (None, None, None)
     slicing...
     True
 
@@ -1806,8 +1813,17 @@ def test_container_proxying():
     [42]
     >>> i.c[5:10]
     slicing...
-    (5, 10)
-    >>> i.c[5:] == (5, sys.maxsize if PY2 else None)
+    (5, 10, None)
+    >>> i.c[5:] == (5, None, None)
+    slicing...
+    True
+    >>> i.c[:10] == (None, 10, None)
+    slicing...
+    True
+    >>> i.c[5:10:5] == (5, 10, 5)
+    slicing...
+    True
+    >>> i.c[::] == (None, None, None)
     slicing...
     True
 
@@ -1820,7 +1836,7 @@ def test_container_proxying():
     ...     def __getitem__(self, key):
     ...         if isinstance(key, slice):
     ...             print('slicing...')
-    ...             return (key.start,key.stop)
+    ...             return (key.start, key.stop, key.step)
     ...         print('getitem', key)
     ...         if key == 4:
     ...             raise IndexError
@@ -1833,7 +1849,7 @@ def test_container_proxying():
     ...         return iter((42,))
     ...     def __getslice__(self, start, end):
     ...         print('slicing...')
-    ...         return (start, end)
+    ...         return (start, end, None)
 
     The naked class behaves like this:
 
@@ -1849,8 +1865,17 @@ def test_container_proxying():
     [42]
     >>> c[5:10]
     slicing...
-    (5, 10)
-    >>> c[5:] == (5, sys.maxsize if PY2 else None)
+    (5, 10, None)
+    >>> c[5:] == (5, None, None)
+    slicing...
+    True
+    >>> c[:10] == (None, 10, None)
+    slicing...
+    True
+    >>> c[5:10:5] == (5, 10, 5)
+    slicing...
+    True
+    >>> c[::] == (None, None, None)
     slicing...
     True
 
@@ -1872,8 +1897,17 @@ def test_container_proxying():
     [42]
     >>> i.c[5:10]
     slicing...
-    (5, 10)
-    >>> i.c[5:] == (5, sys.maxsize if PY2 else None)
+    (5, 10, None)
+    >>> i.c[5:] == (5, None, None)
+    slicing...
+    True
+    >>> i.c[:10] == (None, 10, None)
+    slicing...
+    True
+    >>> i.c[5:10:5] == (5, 10, 5)
+    slicing...
+    True
+    >>> i.c[::] == (None, None, None)
     slicing...
     True
 
@@ -2278,6 +2312,32 @@ class TestBugs(unittest.TestCase):
         except AttributeError:
             raise
 
+    def test_wrapped_attr(self):
+        top = I("")
+        top.f = I("f")
+        f = top.f
+        i = I("i")
+        i.f = f
+        self.assertIs(i.f.aq_self, f)
+        self.assertIs(i.f.aq_parent, i)
+
+    def test__cmp__(self):
+        class CmpOrdered(I):
+            def __cmp__(self, other):
+                return cmp(self.id, other.id)
+        Ordered = CmpOrdered
+        top = Ordered("")
+        top.o = Ordered("")
+        o = top.o
+        self.assertEqual(o.o.__cmp__(top), 0)
+        self.assertEqual(o.o.__cmp__(Ordered("1")), -1)
+
+    def test_bool(self):
+        top = I("")
+        top.o = I("")
+        o = top.o
+        self.assertTrue(bool(o.o))
+
 
 class TestSpecialNames(unittest.TestCase):
 
@@ -2449,6 +2509,43 @@ class TestWrapper(unittest.TestCase):
         # By the time it gets there, it's not wrapped:
         self.assertIs(type(child.__dict__['child2']), Impl)
 
+    def test__bytes__is_correcty_wrapped(self):
+        class A(Implicit):
+
+            def __bytes__(self):
+                return b'my bytes'
+
+        a = A()
+        a.b = A()
+        wrapper = Acquisition.ImplicitAcquisitionWrapper(a.b, a)
+
+        self.assertEqual(b'my bytes', wrapper.__bytes__())
+        self.assertEqual(b'my bytes', bytes(wrapper))
+
+    def test_AttributeError_if_object_has_no__bytes__(self):
+        class A(Implicit):
+            pass
+
+        a = A()
+        a.b = A()
+        wrapper = Acquisition.ImplicitAcquisitionWrapper(a.b, a)
+        with self.assertRaises(AttributeError):
+            wrapper.__bytes__()
+
+        with self.assertRaises(TypeError):
+            bytes(wrapper)
+
+    def test_aq_acquire_with_class_and_filter(self):
+        class C:
+            a = 1
+        allow = lambda *args: True
+        deny = lambda *args: False
+        self.assertEqual(aq_acquire(C, "a", allow), C.a)
+        with self.assertRaises(AttributeError):
+            aq_acquire(C, "a", deny)
+        with self.assertRaises(AttributeError):
+            aq_acquire(C, "b")
+
 
 class TestOf(unittest.TestCase):
 
@@ -2516,7 +2613,7 @@ class TestAQInContextOf(unittest.TestCase):
             def hi(self):
                 return self.color
 
-        class Location(object):
+        class Location:
             __parent__ = None
 
         b = B()
@@ -2609,7 +2706,7 @@ class TestAQInContextOf(unittest.TestCase):
                               Acquisition.ImplicitAcquisitionWrapper)
 
         # Following parent pointers in weird circumstances works too:
-        class WithParent(object):
+        class WithParent:
             __parent__ = None
 
         self.assertEqual(aq_inContextOf(WithParent(), root), 0)
@@ -2637,7 +2734,7 @@ class TestCircles(unittest.TestCase):
         from Acquisition import _Wrapper as Wrapper
         from Acquisition import _Wrapper_acquire
 
-        class Repeated(object):
+        class Repeated:
             hello = "world"
 
             def __repr__(self):
@@ -2765,7 +2862,7 @@ class TestAcquire(unittest.TestCase):
         self.assertEqual(aq_acquire(self.a.b.c, 'nonesuch', default=4), 4)
 
     def test_no_wrapper_but___parent___falls_back_to_default(self):
-        class NotWrapped(object):
+        class NotWrapped:
             pass
         child = NotWrapped()
         child.__parent__ = NotWrapped()
@@ -2789,7 +2886,7 @@ class TestCooperativeBase(unittest.TestCase):
             def __getattribute__(self, name):
                 if name == 'magic':
                     return 42
-                return super(ExtendsBase, self).__getattribute__(name)
+                return super().__getattribute__(name)
 
         class Acquirer(kind, ExtendsBase):
             pass
@@ -2820,7 +2917,7 @@ class TestImplicitWrappingGetattribute(unittest.TestCase):
     @unittest.skipIf(CAPI, 'Pure Python test.')
     def test_object_getattribute_in_rebound_method_with_slots(self):
 
-        class Persistent(object):
+        class Persistent:
             __slots__ = ('__flags',)
 
             def __init__(self):
@@ -2846,7 +2943,7 @@ class TestImplicitWrappingGetattribute(unittest.TestCase):
     @unittest.skipIf(CAPI, 'Pure Python test.')
     def test_type_with_slots_reused(self):
 
-        class Persistent(object):
+        class Persistent:
             __slots__ = ('__flags',)
 
             def __init__(self):
@@ -2864,7 +2961,7 @@ class TestImplicitWrappingGetattribute(unittest.TestCase):
     @unittest.skipIf(CAPI, 'Pure Python test.')
     def test_object_getattribute_in_rebound_method_with_dict(self):
 
-        class Persistent(object):
+        class Persistent:
             def __init__(self):
                 self.__flags = 42
 
@@ -2888,7 +2985,7 @@ class TestImplicitWrappingGetattribute(unittest.TestCase):
     @unittest.skipIf(CAPI, 'Pure Python test.')
     def test_object_getattribute_in_rebound_method_with_slots_and_dict(self):
 
-        class Persistent(object):
+        class Persistent:
             __slots__ = ('__flags', '__dict__')
 
             def __init__(self):
@@ -2964,6 +3061,7 @@ class TestProxying(unittest.TestCase):
     __binary_numeric_methods__ = [
         '__add__',
         '__sub__',
+        '__matmul__',
         '__mul__',
         # '__floordiv__',  # not implemented in C
         '__mod__',
@@ -2995,6 +3093,7 @@ class TestProxying(unittest.TestCase):
         # in place
         '__iadd__',
         '__isub__',
+        '__imatmul__',
         '__imul__',
         '__idiv__',
         '__itruediv__',
@@ -3012,12 +3111,6 @@ class TestProxying(unittest.TestCase):
         # '__coerce__',
     ]
 
-    if PY3 and sys.version_info.minor >= 5:
-        __binary_numeric_methods__.extend([
-            '__matmul__',
-            '__imatmul__'
-        ])
-
     __unary_special_methods__ = [
         # arithmetic
         '__neg__',
@@ -3030,7 +3123,7 @@ class TestProxying(unittest.TestCase):
         # conversion
         '__complex__': complex,
         '__int__': int,
-        '__long__': long,
+        '__long__': int,
         '__float__': float,
         '__oct__': oct,
         '__hex__': hex,
@@ -3062,10 +3155,7 @@ class TestProxying(unittest.TestCase):
             acquire_meths[k] = make_converter(convert)
 
         acquire_meths['__len__'] = lambda self: self.value
-
-        if PY3:
-            # Under Python 3, oct() and hex() call __index__ directly
-            acquire_meths['__index__'] = acquire_meths['__int__']
+        acquire_meths['__index__'] = acquire_meths['__int__']
 
         if base_class == Explicit:
             acquire_meths['value'] = Acquisition.Acquired
@@ -3187,18 +3277,15 @@ class TestProxying(unittest.TestCase):
 
         self.assertEqual(base.derived(1, k=2), (42, 1, 2))
 
-        if not IS_PYPY:
-            # XXX: This test causes certain versions
-            # of PyPy to segfault (at least 2.6.0-alpha1)
-            class NotCallable(base_class):
-                pass
+        class NotCallable(base_class):
+            pass
 
-            base.derived = NotCallable()
-            try:
-                base.derived()
-                self.fail("Not callable")
-            except (TypeError, AttributeError):
-                pass
+        base.derived = NotCallable()
+        try:
+            base.derived()
+            self.fail("Not callable")
+        except (TypeError, AttributeError):
+            pass
 
     def test_implicit_proxy_call(self):
         self._check_call()
@@ -3254,7 +3341,7 @@ class TestProxying(unittest.TestCase):
                             '__ne__', '__ge__', '__le__']
 
         def _never_called(self, other):
-            raise RuntimeError("This should never be called")
+            raise AssertionError("This should never be called")
 
         class RichCmpNeverCalled(base_class):
             for _name in rich_cmp_methods:
@@ -3321,13 +3408,18 @@ class TestProxying(unittest.TestCase):
 
 class TestCompilation(unittest.TestCase):
 
-    def test_compile(self):
-        if IS_PYPY or IS_PURE:
-            with self.assertRaises((AttributeError, ImportError)):
-                from Acquisition import _Acquisition
-        else:
+    def test_compilation(self):
+        self.assertEqual(CAPI, not (IS_PYPY or IS_PURE))
+        try:
             from Acquisition import _Acquisition
+            cExplicit = _Acquisition.Explicit
+        except ImportError:
+            cExplicit = None  # PyPy never has a C module.
+        if CAPI:  # pragma: no cover
             self.assertTrue(hasattr(_Acquisition, 'AcquisitionCAPI'))
+            self.assertEqual(Explicit, cExplicit)
+        else:
+            self.assertNotEqual(Explicit, cExplicit)
 
 
 def test_suite():
